# delimit;
clear;
log using "D:\Dropbox\reimbursement schedules UNH\simulation Reimbursement Paper\simulation", replace t;
set matsize 4000;
set mem 150m;

# delimit;
clear;
set more off;
          
/*******************************************************************/
/* Data Set 1 : uniform health shocks */
/*******************************************************************/
/* assume following specifications for hospitals' decisions*/
/* hopspital's problem: choose to maximize the following: gamma_a*(theta*q^alpha1 + alpha2*theta)   - beta*q  + r(q)
First component: gross utility from paitient's health outcome
second component: costs of treatment
third component: reimbursement from insurer
reimbursement schedule is given by
r(q) = delta1*q      if  q < q1
     = delta1 * q1   if q1 <= q <= q2
     = delta1 * q1 + delta2 * (q - q2) = delta2*q if q > q2 given the way delta2 is determined. 

The corresponding optimal choice of q is outlined in the paper */
    

forvalues k = 1(1)500{;
clear;
set obs 5000;
set seed `k';
gen vol = _N;
generate healthstatus= 100*uniform();
local binno =  vol/ 20;
 
local gamma_a = 5; 
local alpha1 = 0.1; 
local alpha2 = -20; 
local beta = 1;
local delta1 = 0.2;
local delta2 = 0.1;
local q1 =  30;
local q2 =   50 ;

gen q_1st = ((`gamma_a' * `alpha1' * healthstatus) / (`beta'-`delta1' )) ^(1/(1-`alpha1'));
gen q_2nd = ((`gamma_a' * `alpha1' * healthstatus) / `beta' ) ^(1/(1-`alpha1'));
gen q_3rd = ((`gamma_a' * `alpha1' * healthstatus) / (`beta'-`delta2' )) ^(1/(1-`alpha1'));
gen u_1st = `gamma_a' * healthstatus * q_1st^`alpha1' - `beta' * q_1st + `delta1' * q_1st if q_1st < `q1';
replace u_1st = -1000000000000000 if q_1st > `q1';
gen u_2nd = `gamma_a' * healthstatus * q_2nd^`alpha1' - `beta' * q_2nd + `delta1' * `q1' if q_2nd > `q1' & q_2nd < `q2';
replace u_2nd = -1000000000000000 if q_2nd < `q1' | q_2nd > `q2' ;
gen u_3rd = `gamma_a' * healthstatus * q_3rd^`alpha1' - `beta' * q_3rd + `delta1' * `q1' + `delta2' * (q_3rd - `q2')  if q_3rd > `q2';
replace u_3rd = -1000000000000000 if q_3rd < `q2' ;
gen u_kink1 = `gamma_a' * healthstatus * `q1'^`alpha1' - `beta' * `q1' + `delta1' * `q1';
gen u_kink2 = `gamma_a' * healthstatus * `q2'^`alpha1' - `beta' * `q2' + `delta1' * `q1'; 

gen optimalU = max(u_1st,  u_2nd , u_3rd , u_kink1 , u_kink2);
gen qoptimal = q_1st if optimalU  == u_1st;
replace qoptimal = q_2nd if optimalU  == u_2nd;
replace qoptimal = q_3rd if optimalU  == u_3rd;
replace qoptimal = `q1' if optimalU  == u_kink1;
replace qoptimal = `q2' if optimalU  == u_kink2;

gen qhat1 = qoptimal *(1+0.05*(uniform()-0.5));
gen qhat2 = qoptimal *(1+0.1*(uniform()-0.5));
gen qhat3 = qoptimal *(1+0.15*(uniform()-0.5));
gen qhat4 = qoptimal *(1+0.2*(uniform()-0.5));
gen qhat5 = qoptimal *(1+(uniform()<0.1)*0.15*(uniform()-0.5));
gen qhat6 = qoptimal *(1+(uniform()<0.1)*0.20*(uniform()-0.5));
 
 
 
 
gen new_inlier = `delta1';
gen new_outlier = `delta2';
gen new_bpr = `delta1' * `q1';

egen avg = mean( qhat1);
gen ph_billed_3_4 = qhat1 ;
gen inlier_thres = `q1';
gen outlier_thres = `q2';
gen norm_bill_less_outlier = (qhat1 - `q2') / new_bpr; 
gen hospital = 1;


forvalues i = 1(1)6{;

gen sect1_`i' = (qhat`i' < inlier_thres);
gen sect2_`i' = (qhat`i' > inlier_thres & qhat`i' < outlier_thres);
gen sect3_`i' = (qhat`i' >=  outlier_thres);
gen sect_`i' = 1 if sect1_`i' == 1;
replace sect_`i' = 2 if sect2_`i' == 1;
replace sect_`i' = 3 if sect3_`i' == 1;
 

sum qhat`i' if sect_`i' == 2;
gen  qcut`i' = r(max) if sect_`i' == 2; 
sum qhat`i' if sect_`i' == 3;
replace  qcut`i' = r(min) if sect_`i' == 3; 
gen distance_q2_`i' = abs(qhat`i' -  qcut`i');

};


gen sect1_optimal = (qoptimal < inlier_thres);
gen sect2_optimal = (qoptimal > inlier_thres & qoptimal < outlier_thres);
gen sect3_optimal = (qoptimal >=  outlier_thres);
gen sect_optimal = 1 if sect1_optimal == 1;
replace sect_optimal = 2 if sect2_optimal == 1;
replace sect_optimal = 3 if sect3_optimal == 1;

sum qoptimal if sect_optimal == 2;
gen  qcutoptimal = r(max) if sect_optimal == 2; 
sum qoptimal if sect_optimal == 3;
replace  qcutoptimal = r(min) if sect_optimal == 3; 
gen distance_q2_optimal = abs(qoptimal -  qcutoptimal);

  
save "D:\Dropbox\reimbursement schedules UNH\simulation Reimbursement Paper\SimulatedData_`k'", replace;

use  "D:\Dropbox\reimbursement schedules UNH\simulation Reimbursement Paper\SimulatedData_`k'", clear;

sum qoptimal if sect_optimal == 2;
sum qoptimal if sect_optimal == 3;


gen z = qoptimal - `q2';
replace distance_q2_optimal = -1000 if distance_q2_optimal == .;
replace sect_optimal = -1 if sect_optimal == .;
order new_inlier new_outlier new_bpr ph_billed_3_4 inlier_thres outlier_thres norm_bill_less_outlier vol sect1_optimal sect2_optimal sect3_optimal sect_optimal hospital distance_q2_optimal z;
keep new_inlier new_outlier new_bpr ph_billed_3_4 inlier_thres outlier_thres norm_bill_less_outlier vol sect1_optimal sect2_optimal sect3_optimal sect_optimal hospital distance_q2_optimal z;
export delimited using "D:\Dropbox\reimbursement schedules UNH\simulation Reimbursement Paper\SimulatedData_optimal1_`k'.txt", replace  novarnames;
 



forvalues i = 1(1)6{;

use  "D:\Dropbox\reimbursement schedules UNH\simulation Reimbursement Paper\SimulatedData_`k'", clear;

gen z = qhat`i' - `q2';
replace distance_q2_`i' = -1000 if distance_q2_`i' == .;
replace sect_`i' = -1 if sect_`i' == .;
order new_inlier new_outlier new_bpr ph_billed_3_4 inlier_thres outlier_thres norm_bill_less_outlier vol sect1_`i' sect2_`i' sect3_`i' sect_`i' hospital distance_q2_`i'  z ;
keep new_inlier new_outlier new_bpr ph_billed_3_4 inlier_thres outlier_thres norm_bill_less_outlier vol sect1_`i' sect2_`i' sect3_`i' sect_`i' hospital distance_q2_`i' z ;

export delimited using "D:\Dropbox\reimbursement schedules UNH\simulation Reimbursement Paper\SimulatedData`i'_`k'.txt", replace  novarnames;
};  
 
 
 
 
use  "D:\Dropbox\reimbursement schedules UNH\simulation Reimbursement Paper\SimulatedData_`k'", clear;
keep in 1/1000;
save "D:\Dropbox\reimbursement schedules UNH\simulation Reimbursement Paper\SimulatedData_temp1", replace;

sum qoptimal if sect_optimal == 2;
sum qoptimal if sect_optimal == 3;


gen z = qoptimal - `q2';
replace distance_q2_optimal = -1000 if distance_q2_optimal == .;
replace sect_optimal = -1 if sect_optimal == .;
order new_inlier new_outlier new_bpr ph_billed_3_4 inlier_thres outlier_thres norm_bill_less_outlier vol sect1_optimal sect2_optimal sect3_optimal sect_optimal hospital distance_q2_optimal z;
keep new_inlier new_outlier new_bpr ph_billed_3_4 inlier_thres outlier_thres norm_bill_less_outlier vol sect1_optimal sect2_optimal sect3_optimal sect_optimal hospital distance_q2_optimal z;
export delimited using "D:\Dropbox\reimbursement schedules UNH\simulation Reimbursement Paper\SimulatedData_optimal2_`k'.txt", replace  novarnames;



forvalues i = 1(1)6{;

local j = `i' + 6;

use "D:\Dropbox\reimbursement schedules UNH\simulation Reimbursement Paper\SimulatedData_temp1", clear;

gen z = qhat`i' - `q2';
replace distance_q2_`i' = -1000 if distance_q2_`i' == .;
replace sect_`i' = -1 if sect_`i' == .;
order new_inlier new_outlier new_bpr ph_billed_3_4 inlier_thres outlier_thres norm_bill_less_outlier vol sect1_`i' sect2_`i' sect3_`i' sect_`i' hospital distance_q2_`i'  z ;
keep new_inlier new_outlier new_bpr ph_billed_3_4 inlier_thres outlier_thres norm_bill_less_outlier vol sect1_`i' sect2_`i' sect3_`i' sect_`i' hospital distance_q2_`i' z ;

export delimited using "D:\Dropbox\reimbursement schedules UNH\simulation Reimbursement Paper\SimulatedData`j'_`k'.txt", replace  novarnames;
};  
 
 

 
use  "D:\Dropbox\reimbursement schedules UNH\simulation Reimbursement Paper\SimulatedData_`k'", clear;
keep in 1001/1500;
save "D:\Dropbox\reimbursement schedules UNH\simulation Reimbursement Paper\SimulatedData_temp2", replace;

sum qoptimal if sect_optimal == 2;
sum qoptimal if sect_optimal == 3;

gen z = qoptimal - `q2';
replace distance_q2_optimal = -1000 if distance_q2_optimal == .;
replace sect_optimal = -1 if sect_optimal == .;
order new_inlier new_outlier new_bpr ph_billed_3_4 inlier_thres outlier_thres norm_bill_less_outlier vol sect1_optimal sect2_optimal sect3_optimal sect_optimal hospital distance_q2_optimal z;
keep new_inlier new_outlier new_bpr ph_billed_3_4 inlier_thres outlier_thres norm_bill_less_outlier vol sect1_optimal sect2_optimal sect3_optimal sect_optimal hospital distance_q2_optimal z;
export delimited using "D:\Dropbox\reimbursement schedules UNH\simulation Reimbursement Paper\SimulatedData_optimal3_`k'.txt", replace  novarnames;



forvalues i = 1(1)6{;

local j = `i' + 12;

use "D:\Dropbox\reimbursement schedules UNH\simulation Reimbursement Paper\SimulatedData_temp2", clear;

gen z = qhat`i' - `q2';
replace distance_q2_`i' = -1000 if distance_q2_`i' == .;
replace sect_`i' = -1 if sect_`i' == .;
order new_inlier new_outlier new_bpr ph_billed_3_4 inlier_thres outlier_thres norm_bill_less_outlier vol sect1_`i' sect2_`i' sect3_`i' sect_`i' hospital distance_q2_`i'  z ;
keep new_inlier new_outlier new_bpr ph_billed_3_4 inlier_thres outlier_thres norm_bill_less_outlier vol sect1_`i' sect2_`i' sect3_`i' sect_`i' hospital distance_q2_`i' z ;

export delimited using "D:\Dropbox\reimbursement schedules UNH\simulation Reimbursement Paper\SimulatedData`j'_`k'.txt", replace  novarnames;
};  
 
 

use  "D:\Dropbox\reimbursement schedules UNH\simulation Reimbursement Paper\SimulatedData_`k'", clear;
keep in 1501/1600;
save "D:\Dropbox\reimbursement schedules UNH\simulation Reimbursement Paper\SimulatedData_temp3", replace;

sum qoptimal if sect_optimal == 2;
sum qoptimal if sect_optimal == 3;

gen z = qoptimal - `q2';
replace distance_q2_optimal = -1000 if distance_q2_optimal == .;
replace sect_optimal = -1 if sect_optimal == .;
order new_inlier new_outlier new_bpr ph_billed_3_4 inlier_thres outlier_thres norm_bill_less_outlier vol sect1_optimal sect2_optimal sect3_optimal sect_optimal hospital distance_q2_optimal z;
keep new_inlier new_outlier new_bpr ph_billed_3_4 inlier_thres outlier_thres norm_bill_less_outlier vol sect1_optimal sect2_optimal sect3_optimal sect_optimal hospital distance_q2_optimal z;
export delimited using "D:\Dropbox\reimbursement schedules UNH\simulation Reimbursement Paper\SimulatedData_optimal4_`k'.txt", replace  novarnames;


forvalues i = 1(1)6{;

local j = `i' + 18;

use "D:\Dropbox\reimbursement schedules UNH\simulation Reimbursement Paper\SimulatedData_temp3", clear;

gen z = qhat`i' - `q2';
replace distance_q2_`i' = -1000 if distance_q2_`i' == .;
replace sect_`i' = -1 if sect_`i' == .;
order new_inlier new_outlier new_bpr ph_billed_3_4 inlier_thres outlier_thres norm_bill_less_outlier vol sect1_`i' sect2_`i' sect3_`i' sect_`i' hospital distance_q2_`i'  z ;
keep new_inlier new_outlier new_bpr ph_billed_3_4 inlier_thres outlier_thres norm_bill_less_outlier vol sect1_`i' sect2_`i' sect3_`i' sect_`i' hospital distance_q2_`i' z ;

export delimited using "D:\Dropbox\reimbursement schedules UNH\simulation Reimbursement Paper\SimulatedData`j'_`k'.txt", replace  novarnames;
};  
};
 

log close;
