%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% MLE 
%%% Full estimation. estimate both phi2 and phi4
%%% i.e. estimate alpha and beta
%%% use parametric bootstrap instead of nonparametric bootstrap 
%%% using empirical distribution
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%% TABLE 5 (control group): BMT %%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

rng(1);
S = 500;

% load data  
bmtdata_master = xlsread('MLEdata_bmt_control.xls'); % [new_inlier new_outlier new_bpr ph_billed_3_4 inlier_thres outlier_thres norm_bill_less_outlier vol sect1 sect2 sect3 sect hospital,, abs(dis from q2)]
nhospital = max(bmtdata_master(:,8)); % number of hospitals
 
% Create data for LHS regressions (second segment only)
bmt_L_master = bmtdata_master;
bmt_L_master((bmtdata_master(:,7)~=2),:) = [];
  
% Create data for RHS regressions (third segment only)
bmt_R_master = bmtdata_master;
bmt_R_master((bmtdata_master(:,7)~=3),:) = [];

   
% generate Y (q variable)
Ybmt_L_master = bmt_L_master(:,3); 
Ybmt_R_master = bmt_R_master(:,3); 
 
 
% generate X (basis functions in g and lambda): X=[1, delta1, delta2, delta1*q1, q1, q2]
% depending on specification, probably need to include higher order
% polinomials of these
% same X will be used for regression with normalized y 

Xbmt_L_master_alpha = [ones(size(bmt_L_master,1),1), bmt_L_master(:,1:2)]; 
Xbmt_L_master_beta = Xbmt_L_master_alpha;
Xbmt_L_master = [Xbmt_L_master_alpha, Xbmt_L_master_beta];

Xbmt_R_master_alpha = [ones(size(bmt_R_master,1),1), bmt_R_master(:,1:2)]; 
Xbmt_R_master_beta = Xbmt_R_master_alpha;
Xbmt_R_master = [Xbmt_R_master_alpha, Xbmt_R_master_beta];

alphaL_bmt = [-0.5,0.5,1]; % parameters for hazard parameter lambda for L regression 
alphaR_bmt = [-0.5,0.5,1]; % parameters for hazard parameter lambda for R regression
 
betaL_bmt = [-0.5,0.5,1];  % parameters in g1 for L regression
betaR_bmt = [-0.5,0.5,1]; % parameters in g2 for R regression


  

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% L regression, BMT, original Y %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

A = [zeros(size(Xbmt_L_master,1),length(alphaL_bmt)),-Xbmt_L_master_beta; Xbmt_L_master_alpha, zeros(size(Xbmt_L_master,1),length(betaL_bmt));...
    -Xbmt_L_master_alpha, zeros(size(Xbmt_L_master,1),length(betaL_bmt))];
b = [-Ybmt_L_master; 9*ones(size(Xbmt_L_master,1),1); 9*ones(size(Xbmt_L_master,1),1)];
options=optimset('TolCon',1.e-12,'MaxFunEvals',10^4, 'MaxIter',10^4,'LargeScale', 'on','Display', 'off');

[par1_master,val,exit,output] = fmincon(@MLE_bmtL_control,[alphaL_bmt, betaL_bmt],... 
A,b,[],[],[],[],[],options, Ybmt_L_master, Xbmt_L_master);


%model_ll = val
model_exit1 = exit;
 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% R regression, BMT, original Y %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

A = [zeros(size(Xbmt_R_master,1),length(alphaR_bmt)),Xbmt_R_master_beta; Xbmt_R_master_alpha, zeros(size(Xbmt_R_master,1),length(betaR_bmt));...
    -Xbmt_R_master_alpha, zeros(size(Xbmt_R_master,1),length(betaR_bmt))];
b = [Ybmt_R_master; 9*ones(size(Xbmt_R_master,1),1); 9*ones(size(Xbmt_R_master,1),1)];
  

[par2_master,val,exit,output] = fmincon(@MLE_bmtR_control,[alphaR_bmt, betaR_bmt],... 
A,b,[],[],[],[],[],options, Ybmt_R_master, Xbmt_R_master);

%model_ll = val
model_exit2 = exit;


%%%%%%%%%%%%%% parameter estimates from the original sample %%%%%%%%%%%%%%%%%%%%%%%%%%%%%

lambda1_bmt_master = exp(Xbmt_L_master(:,1:length(alphaL_bmt)) * par1_master(1:length(alphaL_bmt))');
lambda2_bmt_master = exp(Xbmt_R_master(:,1:length(alphaR_bmt)) * par2_master(1:length(alphaR_bmt))'); 
gL_master = Xbmt_L_master(:,length(alphaL_bmt)+1:end) * par1_master(length(alphaL_bmt)+1:end)';
gR_master = Xbmt_R_master(:,length(alphaR_bmt)+1:end) * par2_master(length(alphaR_bmt)+1:end)'; 


slopeL_bmt_master_temp = zeros(size(bmt_L_master,1));
gL_bmt_master_temp = zeros(size(bmt_L_master,1));
slopeR_bmt_master_temp = zeros(size(bmt_R_master,1));
gR_bmt_master_temp = zeros(size(bmt_R_master,1));


if model_exit1 >= 1 &  model_exit2 >= 1
slopeL_bmt_master_temp  = 1./lambda1_bmt_master * length(bmtdata_master(:,1)) / length(bmt_L_master(:,1));
gL_bmt_master_temp  = gL_master;
slopeR_bmt_master_temp  = 1./lambda2_bmt_master* length(bmtdata_master(:,1)) / length(bmt_R_master(:,1));
gR_bmt_master_temp  = gR_master;
else
slopeL_bmt_master_temp(:)  = inf;
gL_bmt_master_temp(:)  = inf;
slopeR_bmt_master_temp(:)  = -inf;
gR_bmt_master_temp(:)  = -inf;
end
     

for j = 1:nhospital   
    slopeL_bmt_master(j) = slopeL_bmt_master_temp(find((bmt_L_master(:,8)==j),1));
    gL_bmt_master(j) = gL_bmt_master_temp(find((bmt_L_master(:,8)==j),1));
    slopeR_bmt_master(j) = slopeR_bmt_master_temp(find((bmt_R_master(:,8)==j),1));
    gR_bmt_master(j) = gR_bmt_master_temp(find((bmt_R_master(:,8)==j),1));
end
 
clear  slopeL_bmt_master_temp gL_bmt_master_temp slopeR_bmt_master_temp gR_bmt_master_temp;
    phi4_bmt_master  = slopeR_bmt_master  - slopeL_bmt_master ;
    phi2_bmt_master  = gR_bmt_master  - gL_bmt_master ;
 


if min(phi4_bmt_master) == -inf | max(slopeL_bmt_master) > 300 ... 
                  | max(slopeR_bmt_master) > 300  ...
                | min(phi2_bmt_master) == -inf 
           phi4_bmt_master = [];
           slopeL_bmt_master = [];
           slopeR_bmt_master = [];
           phi2_bmt_master = [];
           gL_bmt_master = [];
           gR_bmt_master = [];           
end

 

%%%%%%%%%%%%%%%%%%%%%%% SIMULATION %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


slopeL_bmt  = zeros(S,nhospital);
gL_bmt  =  zeros(S,nhospital);
slopeR_bmt  = zeros(S,nhospital);
gR_bmt  =  zeros(S,nhospital); 
       
s = 1;
while s <= S;
    
r1 = ceil(size(bmt_L_master,1) .* rand(size(bmt_L_master,1),1));

bmt_L  = bmt_L_master(r1,:);
  
r2 = ceil(size(bmt_R_master,1) .* rand(size(bmt_R_master,1),1));
 
bmt_R = bmt_R_master(r2,:);
     

Xbmt_L_alpha = [ones(size(bmt_L,1),1), bmt_L(:,1:2)]; 
Xbmt_L_beta = Xbmt_L_alpha;
Xbmt_L = [Xbmt_L_alpha, Xbmt_L_beta];

Xbmt_R_alpha = [ones(size(bmt_R,1),1), bmt_R(:,1:2)]; 
Xbmt_R_beta =  Xbmt_R_alpha;
Xbmt_R = [Xbmt_R_alpha, Xbmt_R_beta];


lambda1_bmt = exp(Xbmt_L(:,1:length(alphaL_bmt)) * par1_master(1:length(alphaL_bmt))');
lambda2_bmt = exp(Xbmt_R(:,1:length(alphaR_bmt)) * par2_master(1:length(alphaR_bmt))');
gL = Xbmt_L(:,length(alphaL_bmt)+1:end) * par1_master(length(alphaL_bmt)+1:end)';
gR = Xbmt_R(:,length(alphaR_bmt)+1:end) * par2_master(length(alphaR_bmt)+1:end)'; 
Ybmt_L = zeros(size(bmt_L,1),1);
Ybmt_R = zeros(size(bmt_R,1),1);

%%% f = lambda*exp(-lambda*e)
%%% F = 1-exp(-lambda*e)
%%% invF(f) = -1/lambda *ln(1-f), for L: e = g1-y; for Rl e = y - g2

u = rand(size(bmt_L,1),1);
Ybmt_L = gL  + 1./lambda1_bmt .*log(1-u ); % draw u from uniform, then using the inverse cdf of e, get a random draw of e from its distribution
 
u = rand(size(bmt_R,1),1);
Ybmt_R = gR  - 1./lambda2_bmt .*log(1-u );
 

 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% L regression, BMT, original Y %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

A = [zeros(size(Xbmt_L,1),length(alphaL_bmt)),-Xbmt_L_beta; Xbmt_L_alpha, zeros(size(Xbmt_L,1),length(betaL_bmt));...
    -Xbmt_L_alpha, zeros(size(Xbmt_L,1),length(betaL_bmt))];
b = [-Ybmt_L; 9*ones(size(Xbmt_L,1),1); 9*ones(size(Xbmt_L,1),1)];
options=optimset('TolCon',1.e-12,'MaxFunEvals',10^4, 'MaxIter',10^4,'LargeScale', 'on','Display', 'off');

[par1,val,exit,output] = fmincon(@MLE_bmtL_control,[alphaL_bmt, betaL_bmt],... 
A,b,[],[],[],[],[],options, Ybmt_L, Xbmt_L);


%model_ll = val
model_exit1 = exit;
 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% R regression, BMT, original Y %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

A = [zeros(size(Xbmt_R,1),length(alphaR_bmt)),Xbmt_R_beta; Xbmt_R_alpha, zeros(size(Xbmt_R,1),length(betaR_bmt));...
    -Xbmt_R_alpha, zeros(size(Xbmt_R,1),length(betaR_bmt))];
b = [Ybmt_R; 9*ones(size(Xbmt_R,1),1); 9*ones(size(Xbmt_R,1),1)];
  

[par2,val,exit,output] = fmincon(@MLE_bmtR_control,[alphaR_bmt, betaR_bmt],... 
A,b,[],[],[],[],[],options, Ybmt_R, Xbmt_R);

%model_ll = val
model_exit2 = exit;


lambda1_bmt = exp(Xbmt_L_master(:,1:length(alphaL_bmt)) * par1(1:length(alphaL_bmt))');
lambda2_bmt = exp(Xbmt_R_master(:,1:length(alphaR_bmt)) * par2(1:length(alphaR_bmt))'); 
gL = Xbmt_L_master(:,length(alphaL_bmt)+1:end) * par1(length(alphaL_bmt)+1:end)';
gR = Xbmt_R_master(:,length(alphaR_bmt)+1:end) * par2(length(alphaR_bmt)+1:end)'; 



slopeL_bmt_temp = zeros(size(bmt_L_master,1));
gL_bmt_temp = zeros(size(bmt_L_master,1));
slopeR_bmt_temp = zeros(size(bmt_R_master,1));
gR_bmt_temp = zeros(size(bmt_R_master,1));


if model_exit1 >= 1 &  model_exit2 >= 1
slopeL_bmt_temp = 1./lambda1_bmt * length(bmtdata_master(:,1)) / length(bmt_L_master(:,1));
gL_bmt_temp = gL;
slopeR_bmt_temp = 1./lambda2_bmt * length(bmtdata_master(:,1)) / length(bmt_R_master(:,1));
gR_bmt_temp = gR;
else
slopeL_bmt_temp(:) = inf;
gL_bmt_temp(:) = inf; 
slopeR_bmt_temp(:) = -inf;
gR_bmt_temp(:) = -inf;   
end




for j = 1:nhospital   
    slopeL_bmt(s,j) = slopeL_bmt_temp(find((bmt_L_master(:,8)==j),1));
     gL_bmt(s,j) = gL_bmt_temp(find((bmt_L_master(:,8)==j),1));
     slopeR_bmt(s,j) = slopeR_bmt_temp(find((bmt_R_master(:,8)==j),1));
     gR_bmt(s,j) = gR_bmt_temp(find((bmt_R_master(:,8)==j),1));
end
 
if   max(slopeL_bmt(s,:)) > 30 |  min(slopeL_bmt(s,:)) <  10^(-2) | ... 
     max(slopeR_bmt(s,:)) > 30 | min(slopeR_bmt(s,:)) <  10^(-2) |  ...
      min(gR_bmt(s,:)) < -20 | max(gR_bmt(s,:)) > 20 |   min(gL_bmt(s,:)) < -20 | max(gL_bmt(s,:)) > 20  
 
slopeL_bmt(s,:) = [];
slopeR_bmt(s,:) = []; 
gL_bmt(s,:) = [];
gR_bmt(s,:) = [];      
else
    s = s +1;
end
end



phi4_bmt  = slopeR_bmt  - slopeL_bmt ;
phi2_bmt = gR_bmt  - gL_bmt ;
 

temp_s_bmt  = (phi4_bmt  - ones(size(slopeR_bmt,1),1)*phi4_bmt_master).^2;
 
temp_g_bmt  = (phi2_bmt - ones(size(gL_bmt,1),1)* phi2_bmt_master).^2;
   
 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%% TABLE 5 (control group): KIDNEY %%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
 
 
 
% load data  
kidneydata_master = xlsread('MLEdata_kidney_control.xls'); % [new_inlier new_outlier new_bpr ph_billed_3_4 inlier_thres outlier_thres norm_bill_less_outlier vol sect1 sect2 sect3 sect hospital,, abs(dis from q2)]
nhospital = max(kidneydata_master(:,8)); % number of hospitals
 
% Create data for LHS regressions (second segment only)
kidney_L_master = kidneydata_master;
kidney_L_master((kidneydata_master(:,7)~=2),:) = [];
 
kidney_R_master = kidneydata_master;
kidney_R_master((kidneydata_master(:,7)~=3),:) = [];
 
   
% generate Y (q variable)
Ykidney_L_master = kidney_L_master(:,3); 
Ykidney_R_master = kidney_R_master(:,3); 
 
 
% generate X (basis functions in g and lambda): X=[1, delta1, delta2, delta1*q1, q1, q2]
% depending on specification, probably need to include higher order
% polinomials of these
% same X will be used for regression with normalized y 
 
Xkidney_L_master_alpha = [ones(size(kidney_L_master,1),1), kidney_L_master(:,1:2)]; 
Xkidney_L_master_beta =  Xkidney_L_master_alpha;
Xkidney_L_master = [Xkidney_L_master_alpha, Xkidney_L_master_beta];
 
Xkidney_R_master_alpha = [ones(size(kidney_R_master,1),1), kidney_R_master(:,1:2)]; 
Xkidney_R_master_beta = Xkidney_R_master_alpha ;
Xkidney_R_master = [Xkidney_R_master_alpha, Xkidney_R_master_beta];
 
 
alphaL_kidney = [-3,-3,-3]; % parameters for hazard parameter lambda for L regression 
alphaR_kidney = [-3,-3,-3]; % parameters for hazard parameter lambda for R regression
 
betaL_kidney = [-3,-3,-3];  % parameters in g1 for L regression
betaR_kidney = [-3,-3,-3]; % parameters in g2 for R regression
 
 
 
 
 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% L regression, KIDNEY, original Y %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
A = [zeros(size(Xkidney_L_master,1),length(alphaL_kidney)),-Xkidney_L_master_beta; Xkidney_L_master_alpha, zeros(size(Xkidney_L_master,1),length(betaL_kidney));...
    -Xkidney_L_master_alpha, zeros(size(Xkidney_L_master,1),length(betaL_kidney))];
b = [-Ykidney_L_master; 9*ones(size(Xkidney_L_master,1),1); 9*ones(size(Xkidney_L_master,1),1)];
options=optimset('TolCon',1.e-12,'MaxFunEvals',10^4, 'MaxIter',10^4,'LargeScale', 'on','Display', 'off');
 
[par1_master,val,exit,output] = fmincon(@MLE_kidneyL_control,[alphaL_kidney, betaL_kidney],... 
A,b,[],[],[],[],[],options, Ykidney_L_master, Xkidney_L_master);
 
 
model_exit1 = exit;
 
 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% R regression, KIDNEY, original Y %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
A = [zeros(size(Xkidney_R_master,1),length(alphaR_kidney)),Xkidney_R_master_beta; Xkidney_R_master_alpha, zeros(size(Xkidney_R_master,1),length(betaR_kidney));...
    -Xkidney_R_master_alpha, zeros(size(Xkidney_R_master,1),length(betaR_kidney))];
b = [Ykidney_R_master; 9*ones(size(Xkidney_R_master,1),1); 9*ones(size(Xkidney_R_master,1),1)];
  
 
[par2_master,val,exit,output] = fmincon(@MLE_kidneyR_control,[alphaR_kidney, betaR_kidney],... 
A,b,[],[],[],[],[],options, Ykidney_R_master, Xkidney_R_master);
  
model_exit2 = exit;
 
 
%%%%%%%%%%%%%% parameter estimates from the original sample %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
lambda1_kidney_master = exp(Xkidney_L_master(:,1:length(alphaL_kidney)) * par1_master(1:length(alphaL_kidney))');
lambda2_kidney_master = exp(Xkidney_R_master(:,1:length(alphaR_kidney)) * par2_master(1:length(alphaR_kidney))'); 
gL_master = Xkidney_L_master(:,length(alphaL_kidney)+1:end) * par1_master(length(alphaL_kidney)+1:end)';
gR_master = Xkidney_R_master(:,length(alphaR_kidney)+1:end) * par2_master(length(alphaR_kidney)+1:end)'; 
 
slopeL_kidney_master_temp = zeros(size(kidney_L_master,1));
gL_kidney_master_temp  = zeros(size(kidney_L_master,1));
slopeR_kidney_master_temp  = zeros(size(kidney_R_master,1));
gR_kidney_master_temp = zeros(size(kidney_R_master,1));

if model_exit1 >= 1 &  model_exit2 >= 1 
slopeL_kidney_master_temp = 1./lambda1_kidney_master * length(kidneydata_master(:,1)) / length(kidney_L_master(:,1)) ;
gL_kidney_master_temp =  gL_master;
slopeR_kidney_master_temp = 1./lambda2_kidney_master* length(kidneydata_master(:,1)) / length(kidney_R_master(:,1)) ;
gR_kidney_master_temp =  gR_master;
else
slopeL_kidney_master_temp(:) = inf;
gL_kidney_master_temp(:) = inf;
slopeR_kidney_master_temp(:) = -inf;
gR_kidney_master_temp(:) = -inf;
end



for j = 1:nhospital   
     slopeL_kidney_master(j) = slopeL_kidney_master_temp(find((kidney_L_master(:,8)==j),1));
     gL_kidney_master(j) = gL_kidney_master_temp(find((kidney_L_master(:,8)==j),1));
     slopeR_kidney_master(j) = slopeR_kidney_master_temp(find((kidney_R_master(:,8)==j),1));
     gR_kidney_master(j) = gR_kidney_master_temp(find((kidney_R_master(:,8)==j),1));
end

clear slopeL_kidney_master_temp gL_kidney_master_temp slopeR_kidney_master_temp gR_kidney_master_temp; 

    phi4_kidney_master = slopeR_kidney_master - slopeL_kidney_master ;
    phi2_kidney_master = gR_kidney_master - gL_kidney_master ;
 
 
 
if min(phi4_kidney_master) == -inf | max(slopeL_kidney_master) > 300 ... 
                | max(slopeR_kidney_master) > 300   ...
                | min(phi2_kidney_master) == -inf 
           phi4_kidney_master = [];
           slopeL_kidney_master = [];
           slopeR_kidney_master = [];
           phi2_kidney_master = [];
           gL_kidney_master = [];
           gR_kidney_master = [];           
end
 
 
 
%%%%%%%%%%%%%%%%%%%%%%% SIMULATION %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 

slopeL_kidney  = zeros(S,nhospital);
gL_kidney  =  zeros(S,nhospital);
slopeR_kidney  = zeros(S,nhospital);
gR_kidney  =  zeros(S,nhospital); 

s = 1;
while s <= S  
     
r1 = ceil(size(kidney_L_master,1) .* rand(size(kidney_L_master,1),1));

kidney_L  = kidney_L_master(r1,:);
  
r2 = ceil(size(kidney_R_master,1) .* rand(size(kidney_R_master,1),1));
 
kidney_R = kidney_R_master(r2,:);
 
Xkidney_L_alpha = [ones(size(kidney_L,1),1), kidney_L(:,1:2)]; 
Xkidney_L_beta = Xkidney_L_alpha;
Xkidney_L = [Xkidney_L_alpha, Xkidney_L_beta];
 
Xkidney_R_alpha = [ones(size(kidney_R,1),1), kidney_R(:,1:2)]; 
Xkidney_R_beta =  Xkidney_R_alpha;
Xkidney_R = [Xkidney_R_alpha, Xkidney_R_beta];
 
 
lambda1_kidney = exp(Xkidney_L(:,1:length(alphaL_kidney)) * par1_master(1:length(alphaL_kidney))');
lambda2_kidney = exp(Xkidney_R(:,1:length(alphaR_kidney)) * par2_master(1:length(alphaR_kidney))');
gL = Xkidney_L(:,length(alphaL_kidney)+1:end) * par1_master(length(alphaL_kidney)+1:end)';
gR = Xkidney_R(:,length(alphaR_kidney)+1:end) * par2_master(length(alphaR_kidney)+1:end)'; 
Ykidney_L = zeros(size(kidney_L,1),1);
Ykidney_R = zeros(size(kidney_R,1),1);
 
%%% f = lambda*exp(-lambda*e)
%%% F = 1-exp(-lambda*e)
%%% invF(e) = -1/lambda *ln(1-e), for L: e = g1-y; for Rl e = y - g2
 
u = rand(size(kidney_L,1),1);
  
    Ykidney_L = gL  + 1./lambda1_kidney .*log(1-u ); % draw u from uniform, then using the inverse cdf of e, get a random draw of e from its distribution
 
 
u = rand(size(kidney_R,1),1);
  
    Ykidney_R = gR  - 1./lambda2_kidney .*log(1-u );
  
 
 
 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% L regression, KIDNEY, original Y %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
A = [zeros(size(Xkidney_L,1),length(alphaL_kidney)),-Xkidney_L_beta; Xkidney_L_alpha, zeros(size(Xkidney_L,1),length(betaL_kidney));...
    -Xkidney_L_alpha, zeros(size(Xkidney_L,1),length(betaL_kidney))];
b = [-Ykidney_L; 9*ones(size(Xkidney_L,1),1); 9*ones(size(Xkidney_L,1),1)];
options=optimset('TolCon',1.e-12,'MaxFunEvals',10^4, 'MaxIter',10^4,'LargeScale', 'on','Display', 'off');
 
[par1,val,exit,output] = fmincon(@MLE_kidneyL_control,[alphaL_kidney, betaL_kidney],... 
A,b,[],[],[],[],[],options, Ykidney_L, Xkidney_L);
 
 
%model_ll = val
model_exit1 = exit;
 
 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% R regression, KIDNEY, original Y %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
A = [zeros(size(Xkidney_R,1),length(alphaR_kidney)),Xkidney_R_beta; Xkidney_R_alpha, zeros(size(Xkidney_R,1),length(betaR_kidney));...
    -Xkidney_R_alpha, zeros(size(Xkidney_R,1),length(betaR_kidney))];
b = [Ykidney_R; 9*ones(size(Xkidney_R,1),1); 9*ones(size(Xkidney_R,1),1)];
  
 
[par2,val,exit,output] = fmincon(@MLE_kidneyR_control,[alphaR_kidney, betaR_kidney],... 
A,b,[],[],[],[],[],options, Ykidney_R, Xkidney_R);
  
model_exit2 = exit;
 
 
lambda1_kidney = exp(Xkidney_L_master(:,1:length(alphaL_kidney)) * par1(1:length(alphaL_kidney))');
lambda2_kidney = exp(Xkidney_R_master(:,1:length(alphaR_kidney)) * par2(1:length(alphaR_kidney))'); 
gL = Xkidney_L_master(:,length(alphaL_kidney)+1:end) * par1(length(alphaL_kidney)+1:end)';
gR = Xkidney_R_master(:,length(alphaR_kidney)+1:end) * par2(length(alphaR_kidney)+1:end)'; 
 
 
slopeL_kidney_temp = zeros(size(kidney_L_master,1));
gL_kidney_temp = zeros(size(kidney_L_master,1));
slopeR_kidney_temp = zeros(size(kidney_R_master,1));
gR_kidney_temp = zeros(size(kidney_R_master,1));

if model_exit1 >= 1 &  model_exit2 >= 1
slopeL_kidney_temp = 1./lambda1_kidney * length(kidneydata_master(:,1)) / length(kidney_L_master(:,1));
gL_kidney_temp = gL; 
slopeR_kidney_temp = 1./lambda2_kidney * length(kidneydata_master(:,1)) / length(kidney_R_master(:,1));
gR_kidney_temp = gR;
else
slopeL_kidney_temp(:) = inf;
gL_kidney_temp(:) = inf;   
slopeR_kidney_temp(:) = -inf;
gR_kidney_temp(:) = -inf;  
end


for j = 1:nhospital   
     slopeL_kidney(s,j) = slopeL_kidney_temp(find((kidney_L_master(:,8)==j),1));
     gL_kidney(s,j) = gL_kidney_temp(find((kidney_L_master(:,8)==j),1));
     slopeR_kidney(s,j) = slopeR_kidney_temp(find((kidney_R_master(:,8)==j),1));
     gR_kidney(s,j) = gR_kidney_temp(find((kidney_R_master(:,8)==j),1));
end
 

if   max(slopeL_kidney(s,:)) > 30 |  min(slopeL_kidney(s,:)) <  10^(-2) | ... 
     max(slopeR_kidney(s,:)) > 30 | min(slopeR_kidney(s,:)) <  10^(-2) |  ...
      min(gR_kidney(s,:)) < -20 | max(gR_kidney(s,:)) > 20 |   min(gL_kidney(s,:)) < -20 | max(gL_kidney(s,:)) > 20      

 
 
           slopeL_kidney(s,:) = [];
            slopeR_kidney(s,:) = []; 
            gL_kidney(s,:) = [];
             gR_kidney(s,:) = [];      
else
    s=s+1;
end

end


phi4_kidney  = slopeR_kidney  - slopeL_kidney ;
phi2_kidney  = gR_kidney  - gL_kidney ;
  
temp_s_kidney  = (phi4_kidney  - ones(size(slopeR_kidney,1),1)*phi4_kidney_master).^2;
temp_g_kidney  = (phi2_kidney - ones(size(gL_kidney,1),1)* phi2_kidney_master).^2;
 
 

 
 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%% TABLE 5 (control group): LIVER %%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
 
 
 
% load data  
liverdata_master = xlsread('MLEdata_liver_control.xls'); % [new_inlier new_outlier new_bpr ph_billed_3_4 inlier_thres outlier_thres norm_bill_less_outlier vol sect1 sect2 sect3 sect hospital,, abs(dis from q2)]
nhospital = max(liverdata_master(:,8)); % number of hospitals
 
% Create data for LHS regressions (second segment only)
liver_L_master = liverdata_master;
liver_L_master((liverdata_master(:,7) ~= 2),:) = [];
  

liver_R_master = liverdata_master;
liver_R_master((liverdata_master(:,7) ~= 3),:) = [];
 
  
% generate Y (q variable)
Yliver_L_master = liver_L_master(:,3); 
Yliver_R_master = liver_R_master(:,3); 
 
 
% generate X (basis functions in g and lambda): X=[1, delta1, delta2, delta1*q1, q1, q2]
% depending on specification, probably need to include higher order
% polinomials of these
% same X will be used for regression with normalized y 
 
Xliver_L_master_alpha = [ones(size(liver_L_master,1),1), liver_L_master(:,1:2) ]; 
Xliver_L_master_beta =  Xliver_L_master_alpha ;
Xliver_L_master = [Xliver_L_master_alpha, Xliver_L_master_beta];
 
Xliver_R_master_alpha = [ones(size(liver_R_master,1),1), liver_R_master(:,1:2) ]; 
Xliver_R_master_beta =  Xliver_R_master_alpha ;
Xliver_R_master = [Xliver_R_master_alpha, Xliver_R_master_beta];

 
 
alphaL_liver = [-3,-3,-3]; % parameters for hazard parameter lambda for L regression 
alphaR_liver = [-3,-3,-3]; % parameters for hazard parameter lambda for R regression
 
betaL_liver = [-3,-3,-3];  % parameters in g1 for L regression
betaR_liver = [-3,-3,-3]; % parameters in g2 for R regression
 
 
 
 
 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% L regression, LIVER, original Y %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
A = [zeros(size(Xliver_L_master,1),length(alphaL_liver)),-Xliver_L_master_beta; Xliver_L_master_alpha, zeros(size(Xliver_L_master,1),length(betaL_liver));...
    -Xliver_L_master_alpha, zeros(size(Xliver_L_master,1),length(betaL_liver))];
b = [-Yliver_L_master; 9*ones(size(Xliver_L_master,1),1); 9*ones(size(Xliver_L_master,1),1)];
options=optimset('TolCon',1.e-12,'MaxFunEvals',10^4, 'MaxIter',10^4,'LargeScale', 'on','Display', 'off');
 
[par1_master,val,exit,output] = fmincon(@MLE_liverL_control,[alphaL_liver, betaL_liver],... 
A,b,[],[],[],[],[],options, Yliver_L_master, Xliver_L_master);
 
 
%model_ll = val
model_exit1 = exit;
 
 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% R regression, LIVER, original Y %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
A = [zeros(size(Xliver_R_master,1),length(alphaR_liver)),Xliver_R_master_beta; Xliver_R_master_alpha, zeros(size(Xliver_R_master,1),length(betaR_liver));...
    -Xliver_R_master_alpha, zeros(size(Xliver_R_master,1),length(betaR_liver))];
b = [Yliver_R_master; 9*ones(size(Xliver_R_master,1),1); 9*ones(size(Xliver_R_master,1),1)];
  
 
[par2_master,val,exit,output] = fmincon(@MLE_liverR_control,[alphaR_liver, betaR_liver],... 
A,b,[],[],[],[],[],options, Yliver_R_master, Xliver_R_master);
  
model_exit2 = exit;
 
 
%%%%%%%%%%%%%% parameter estimates from the original sample %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
lambda1_liver_master = exp(Xliver_L_master(:,1:length(alphaL_liver)) * par1_master(1:length(alphaL_liver))');
lambda2_liver_master = exp(Xliver_R_master(:,1:length(alphaR_liver)) * par2_master(1:length(alphaR_liver))'); 
gL_master = Xliver_L_master(:,length(alphaL_liver)+1:end) * par1_master(length(alphaL_liver)+1:end)';
gR_master = Xliver_R_master(:,length(alphaR_liver)+1:end) * par2_master(length(alphaR_liver)+1:end)'; 
 

slopeL_liver_master_temp = zeros(size(liver_L_master,1));
gL_liver_master_temp  = zeros(size(liver_L_master,1));
slopeR_liver_master_temp  = zeros(size(liver_R_master,1));
gR_liver_master_temp = zeros(size(liver_R_master,1));


if model_exit1 >= 1 &  model_exit2 >= 1
slopeL_liver_master_temp =    1./lambda1_liver_master* length(liverdata_master(:,1)) / length(liver_L_master(:,1));
gL_liver_master_temp = gL_master;
slopeR_liver_master_temp =    1./lambda2_liver_master* length(liverdata_master(:,1)) / length(liver_R_master(:,1));
gR_liver_master_temp = gR_master;
 else
slopeL_liver_master_temp(:) = inf;
gL_liver_master_temp(:) = inf;
slopeR_liver_master_temp(:) = -inf;
gR_liver_master_temp(:) = -inf;
 end
      
for j = 1:nhospital   
     slopeL_liver_master(j) = slopeL_liver_master_temp(find((liver_L_master(:,8)==j),1));
     gL_liver_master(j) = gL_liver_master_temp(find((liver_L_master(:,8)==j),1));
     slopeR_liver_master(j) = slopeR_liver_master_temp(find((liver_R_master(:,8)==j),1));
     gR_liver_master(j) = gR_liver_master_temp(find((liver_R_master(:,8)==j),1));
end
  
clear  slopeL_liver_master_temp gL_liver_master_temp    slopeR_liver_master_temp gR_liver_master_temp ;
  
    phi4_liver_master  = slopeR_liver_master - slopeL_liver_master ;
    phi2_liver_master  = gR_liver_master  - gL_liver_master ;
 
 
 
if min(phi4_liver_master) == -inf | max(slopeL_liver_master) > 300 ... 
                | max(slopeR_liver_master) > 300  ...
                | min(phi2_liver_master) == -inf 
           phi4_liver_master = [];
           slopeL_liver_master = [];
           slopeR_liver_master = [];
           phi2_liver_master = [];
           gL_liver_master = [];
           gR_liver_master = [];           
end
 
 
 
%%%%%%%%%%%%%%%%%%%%%%% SIMULATION %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 

slopeL_liver  = zeros(S,nhospital);
gL_liver  =  zeros(S,nhospital);
slopeR_liver  = zeros(S,nhospital);
gR_liver  =  zeros(S,nhospital); 

s= 1;
while s <=S
    
 
r1 = ceil(size(liver_L_master,1) .* rand(size(liver_L_master,1),1));

liver_L  = liver_L_master(r1,:);
  
r2 = ceil(size(liver_R_master,1) .* rand(size(liver_R_master,1),1));
 
liver_R = liver_R_master(r2,:);
    

Xliver_L_alpha = [ones(size(liver_L,1),1), liver_L(:,1:2)]; 
Xliver_L_beta =  Xliver_L_alpha;
Xliver_L = [Xliver_L_alpha, Xliver_L_beta];
 
Xliver_R_alpha = [ones(size(liver_R,1),1), liver_R(:,1:2)]; 
Xliver_R_beta = Xliver_R_alpha;
Xliver_R = [Xliver_R_alpha, Xliver_R_beta];
 
 
lambda1_liver = exp(Xliver_L(:,1:length(alphaL_liver)) * par1_master(1:length(alphaL_liver))');
lambda2_liver = exp(Xliver_R(:,1:length(alphaR_liver)) * par2_master(1:length(alphaR_liver))');
gL = Xliver_L(:,length(alphaL_liver)+1:end) * par1_master(length(alphaL_liver)+1:end)';
gR = Xliver_R(:,length(alphaR_liver)+1:end) * par2_master(length(alphaR_liver)+1:end)'; 
Yliver_L = zeros(size(liver_L,1),1);
Yliver_R = zeros(size(liver_R,1),1);
 
%%% f = lambda*exp(-lambda*e)
%%% F = 1-exp(-lambda*e)
%%% invF(e) = -1/lambda *ln(1-e), for L: e = g1-y; for Rl e = y - g2
 
u = rand(size(liver_L,1),1);
Yliver_L = gL  + 1./lambda1_liver .*log(1-u ); % draw u from uniform, then using the inverse cdf of e, get a random draw of e from its distribution

 
u = rand(size(liver_R,1),1);
Yliver_R = gR  - 1./lambda2_liver .*log(1-u );
  
 
 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% L regression, LIVER, original Y %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
A = [zeros(size(Xliver_L,1),length(alphaL_liver)),-Xliver_L_beta; Xliver_L_alpha, zeros(size(Xliver_L,1),length(betaL_liver));...
    -Xliver_L_alpha, zeros(size(Xliver_L,1),length(betaL_liver))];
b = [-Yliver_L; 9*ones(size(Xliver_L,1),1); 9*ones(size(Xliver_L,1),1)];
options=optimset('TolCon',1.e-12,'MaxFunEvals',10^4, 'MaxIter',10^4,'LargeScale', 'on','Display', 'off');
 
[par1,val,exit,~] = fmincon(@MLE_liverL_control,[alphaL_liver, betaL_liver],... 
A,b,[],[],[],[],[],options, Yliver_L, Xliver_L);
 
  
model_exit1 = exit;
 
 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% R regression, LIVER, original Y %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
A = [zeros(size(Xliver_R,1),length(alphaR_liver)),Xliver_R_beta; Xliver_R_alpha, zeros(size(Xliver_R,1),length(betaR_liver));...
    -Xliver_R_alpha, zeros(size(Xliver_R,1),length(betaR_liver))];
b = [Yliver_R; 9*ones(size(Xliver_R,1),1); 9*ones(size(Xliver_R,1),1)];
  
 
[par2,val,exit,output] = fmincon(@MLE_liverR_control,[alphaR_liver, betaR_liver],... 
A,b,[],[],[],[],[],options, Yliver_R, Xliver_R);
  
model_exit2 = exit;
 
 
lambda1_liver = exp(Xliver_L_master(:,1:length(alphaL_liver)) * par1(1:length(alphaL_liver))');
lambda2_liver = exp(Xliver_R_master(:,1:length(alphaR_liver)) * par2(1:length(alphaR_liver))'); 
gL = Xliver_L_master(:,length(alphaL_liver)+1:end) * par1(length(alphaL_liver)+1:end)';
gR = Xliver_R_master(:,length(alphaR_liver)+1:end) * par2(length(alphaR_liver)+1:end)'; 
 

 
slopeL_liver_temp = zeros(size(liver_L_master,1));
gL_liver_temp = zeros(size(liver_L_master,1));
slopeR_liver_temp = zeros(size(liver_R_master,1));
gR_liver_temp = zeros(size(liver_R_master,1));

if model_exit1 >= 1 &  model_exit2 >= 1 
slopeL_liver_temp = 1./lambda1_liver * length(liverdata_master(:,1)) / length(liver_L_master(:,1));
gL_liver_temp = gL; 
slopeR_liver_temp = 1./lambda2_liver * length(liverdata_master(:,1)) / length(liver_R_master(:,1));
gR_liver_temp = gR; 
else
slopeL_liver_temp(:) = inf;
gL_liver_temp(:) = inf;   
slopeR_liver_temp(:) = -inf;
gR_liver_temp(:) = -inf;                
end
 
for j = 1:nhospital   
     slopeL_liver(s,j) = slopeL_liver_temp(find((liver_L_master(:,8)==j),1));
     gL_liver(s,j) = gL_liver_temp(find((liver_L_master(:,8)==j),1));
     slopeR_liver(s,j) = slopeR_liver_temp(find((liver_R_master(:,8)==j),1));
     gR_liver(s,j) = gR_liver_temp(find((liver_R_master(:,8)==j),1));
end 
 

if   max(slopeL_liver(s,:)) > 30 |  min(slopeL_liver(s,:)) <  10^(-2) | ... 
     max(slopeR_liver(s,:)) > 30 | min(slopeR_liver(s,:)) <  10^(-2) |  ...
      min(gR_liver(s,:)) < -20 | max(gR_liver(s,:)) > 20 |   min(gL_liver(s,:)) < -20 | max(gL_liver(s,:)) > 20   
 
           slopeL_liver(s,:) = [];
            slopeR_liver(s,:) = []; 
            gL_liver(s,:) = [];
             gR_liver(s,:) = [];          
 
else
    s = s+1;
end
end
 
 
 
phi4_liver  = slopeR_liver  - slopeL_liver ;
phi2_liver = gR_liver  - gL_liver ;
  
temp_s_liver  = (phi4_liver  - ones(size(slopeR_liver,1),1)*phi4_liver_master).^2;
temp_g_liver  = (phi2_liver - ones(size(gL_liver,1),1)* phi2_liver_master).^2;

 
  
sL = slopeL_bmt_master' * 100%LHS slope for each hospital (slope in the quantile function)   
sR = slopeR_bmt_master' * 100% RHS slope for each hospital (slope in the quantile function)
phi4 = phi4_bmt_master'   * 100  % phi4 estiamtes for each hospital 
se_phi4_bmt = sqrt(sum(temp_s_bmt,1)./(size(temp_s_bmt,1)-1))' * 100% parametric boostrapped se of phi4
t_phi4_bmt = phi4_bmt_master'* 100 ./ se_phi4_bmt 
phi2 = (2*phi2_bmt_master'  - mean(phi2_bmt',2))* 100
CImatrix_bmt_phi2 = 2*phi2_bmt_master'* ones(1,size(phi2_bmt',2)) - phi2_bmt';
CIlower_bmt_phi2 = quantile(CImatrix_bmt_phi2, .025,2) * 100
% due to asymptotic non-normality of gap estimates, 
% need to make some adjustments to CI, and also need to report CI instead of se 
CIupper_bmt_phi2 = quantile(CImatrix_bmt_phi2, .975,2)* 100



 


sL = slopeL_kidney_master' * 100%LHS slope for each hospital (slope in the quantile function)   
sR = slopeR_kidney_master' * 100% RHS slope for each hospital (slope in the quantile function)
phi4 = phi4_kidney_master'   * 100  % phi4 estiamtes for each hospital 
se_phi4_kidney = sqrt(sum(temp_s_kidney,1)./(size(temp_s_kidney,1)-1))'* 100 % parametric boostrapped se of phi4
t_phi4_kidney = phi4_kidney_master' * 100./ se_phi4_kidney 
phi2 = (2*phi2_kidney_master'  - mean(phi2_kidney',2))* 100
CImatrix_kidney_phi2 = 2*phi2_kidney_master'* ones(1,size(phi2_kidney',2)) - phi2_kidney';
CIlower_kidney_phi2 = quantile(CImatrix_kidney_phi2, .025,2) * 100
% due to asymptotic non-normality of gap estimates, 
% need to make some adjustments to CI, and also need to report CI instead of se 
CIupper_kidney_phi2 = quantile(CImatrix_kidney_phi2, .975,2)* 100

 


 
sL = slopeL_liver_master'* 100 %LHS slope for each hospital (slope in the quantile function)   
sR = slopeR_liver_master'* 100% RHS slope for each hospital (slope in the quantile function)
phi4 = phi4_liver_master'   * 100  % phi4 estiamtes for each hospital 
se_phi4_liver = sqrt(sum(temp_s_liver,1)./(size(temp_s_liver,1)-1))' * 100% parametric boostrapped se of phi4
t_phi4_liver = phi4_liver_master'* 100 ./ se_phi4_liver 
 
phi2 = (2*phi2_liver_master'  - mean(phi2_liver',2))* 100
CImatrix_liver_phi2 = 2*phi2_liver_master'* ones(1,size(phi2_liver',2)) - phi2_liver';
CIlower_liver_phi2 = quantile(CImatrix_liver_phi2, .025,2) * 100
% due to asymptotic non-normality of gap estimates, 
% need to make some adjustments to CI, and also need to report CI instead of se 
CIupper_liver_phi2 = quantile(CImatrix_liver_phi2, .975,2)* 100

 

